@@ -2,6 +2,7 @@ from ..timeoutException import TimeoutException
 from .tradeException import TradeException
 from ..errorHandler import ValidationException, NotFoundException, InternalException, UnauthorizedException, \
     TooManyRequestsException
+from ..optionsValidator import OptionsValidator
 from .notSynchronizedException import NotSynchronizedException
 from .notConnectedException import NotConnectedException
 from .synchronizationListener import SynchronizationListener
@@ -36,27 +37,37 @@ class MetaApiWebsocketClient:
             token: Authorization token.
             opts: Websocket client options.
         """
+        validator = OptionsValidator()
         opts = opts or {}
-        opts['packetOrderingTimeout'] = opts['packetOrderingTimeout'] if 'packetOrderingTimeout' in opts else 60
+        opts['packetOrderingTimeout'] = validator.validate_non_zero(
+            opts['packetOrderingTimeout'] if 'packetOrderingTimeout' in opts else None, 60, 'packetOrderingTimeout')
         opts['synchronizationThrottler'] = opts['synchronizationThrottler'] if 'synchronizationThrottler' in \
                                                                                opts else {}
         self._httpClient = http_client
         self._application = opts['application'] if 'application' in opts else 'MetaApi'
         self._domain = opts["domain"] if "domain" in opts else "agiliumtrade.agiliumtrade.ai"
         self._url = f'https://mt-client-api-v1.{self._domain}'
-        self._request_timeout = opts['requestTimeout'] if 'requestTimeout' in opts else 60
-        self._connect_timeout = opts['connectTimeout'] if 'connectTimeout' in opts else 60
+        self._request_timeout = validator.validate_non_zero(
+            opts['requestTimeout'] if 'requestTimeout' in opts else None, 60, 'requestTimeout')
+        self._connect_timeout = validator.validate_non_zero(
+            opts['connectTimeout'] if 'connectTimeout' in opts else None, 60, 'connectTimeout')
         retry_opts = opts['retryOpts'] if 'retryOpts' in opts else {}
-        self._retries = retry_opts['retries'] if 'retries' in retry_opts else 5
-        self._minRetryDelayInSeconds = retry_opts['minDelayInSeconds'] if 'minDelayInSeconds' in retry_opts else 1
-        self._maxRetryDelayInSeconds = retry_opts['maxDelayInSeconds'] if 'maxDelayInSeconds' in retry_opts else 30
+        self._retries = validator.validate_number(
+            retry_opts['retries'] if 'retries' in retry_opts else None, 5, 'retries')
+        self._minRetryDelayInSeconds = validator.validate_non_zero(
+            retry_opts['minDelayInSeconds'] if 'minDelayInSeconds' in retry_opts else None, 1, 'minDelayInSeconds')
+        self._maxRetryDelayInSeconds = validator.validate_non_zero(
+            retry_opts['maxDelayInSeconds'] if 'maxDelayInSeconds' in retry_opts else None, 30, 'maxDelayInSeconds')
         self._maxAccountsPerInstance = 100
-        self._subscribeCooldownInSeconds = retry_opts['subscribeCooldownInSeconds'] if 'subscribeCooldownInSeconds' \
-            in retry_opts else 600
+        self._subscribeCooldownInSeconds = validator.validate_non_zero(
+            retry_opts['subscribeCooldownInSeconds'] if 'subscribeCooldownInSeconds' in retry_opts else None, 600,
+            'subscribeCooldownInSeconds')
         event_processing = opts['eventProcessing'] if 'eventProcessing' in opts else {}
-        self._sequentialEventProcessing = event_processing['sequentialProcessing'] if \
-            'sequentialProcessing' in event_processing else False
-        self._useSharedClientApi = opts['useSharedClientApi'] if 'useSharedClientApi' in opts else False
+        self._sequentialEventProcessing = validator.validate_boolean(
+            event_processing['sequentialProcessing'] if 'sequentialProcessing' in event_processing else None, False,
+            'sequentialProcessing')
+        self._useSharedClientApi = validator.validate_boolean(
+            opts['useSharedClientApi'] if 'useSharedClientApi' in opts else None, False, 'useSharedClientApi')
         self._token = token
         self._synchronizationListeners = {}
         self._latencyListeners = []
@@ -197,12 +208,21 @@ class MetaApiWebsocketClient:
         socket_instance = instance['socket']
         self._socketInstances.append(instance)
         instance['connected'] = True
+        first_connect = True
         if len(self._socketInstances) == 1:
             self._packetOrderer.start()
 
         @socket_instance.on('connect')
         async def on_connect():
-            print(f'[{datetime.now().isoformat()}] MetaApi websocket client connected to the MetaApi server')
+            is_shared_client_api = socket_instance.connection_url.split("?")[0] == self._url
+            print(f'[{datetime.now().isoformat()}] MetaApi websocket client connected to the MetaApi server via '
+                  f'{socket_instance.connection_url.split("?")[0]} '
+                  f'{"shared" if is_shared_client_api else "dedicated"} server')
+            nonlocal first_connect
+            if socket_instance_index == 0 and first_connect and not is_shared_client_api:
+                print('Please note that it can take up to 3 minutes for your dedicated server to start for the '
+                      'first time. During this time it is OK if you see some connection errors.')
+                first_connect = False
             if not instance['resolved']:
                 instance['resolved'] = True
                 instance['connectResult'].set_result(None)
@@ -872,8 +892,7 @@ class MetaApiWebsocketClient:
         account_id = packet['accountId']
         packets = self._packetOrderer.restore_order(packet)
         if self._sequentialEventProcessing:
-            events = list(map(lambda packet: asyncio.wait({
-                asyncio.create_task(self._process_synchronization_packet(packet))}), packets))
+            events = list(map(lambda packet: self._process_synchronization_packet(packet), packets))
             if account_id not in self._eventQueues:
                 self._eventQueues[account_id] = deque(events)
                 asyncio.create_task(self._call_account_events(account_id))
@@ -997,6 +1016,8 @@ class MetaApiWebsocketClient:
                     retry_counter += 1
                 else:
                     raise err
+                if account_id not in self._socketInstancesByAccounts:
+                    raise err
             except Exception as err:
                 if err.__class__.__name__ in ['NotSynchronizedException', 'TimeoutException',
                                               'NotAuthenticatedException', 'InternalException'] and retry_counter < \
@@ -1006,6 +1027,8 @@ class MetaApiWebsocketClient:
                     retry_counter += 1
                 else:
                     raise err
+                if account_id not in self._socketInstancesByAccounts:
+                    raise err
 
     async def _make_request(self, account_id: str, request: dict, timeout_in_seconds: float = None):
         socket_instance = self._socketInstances[self._socketInstancesByAccounts[account_id]]
@@ -1099,7 +1122,7 @@ class MetaApiWebsocketClient:
                     await asyncio.sleep(60)
                     if is_only_active_instance():
                         self._subscriptionManager.on_timeout(data["accountId"], instance_number)
-                    self.queue_event(data["accountId"], asyncio.wait({asyncio.create_task(on_disconnected(True))}))
+                    self.queue_event(data["accountId"], on_disconnected(True))
 
                 cancel_disconnect_timer()
                 self._status_timers[instance_id] = asyncio.create_task(disconnect())
@@ -1690,8 +1713,7 @@ class MetaApiWebsocketClient:
                         print(f'[{datetime.now().isoformat()}] Failed to notify reconnect listener',
                               format_error(err))
 
-                self.queue_event(listener['accountId'],
-                                 asyncio.wait({asyncio.create_task(on_reconnected_task(listener))}))
+                self.queue_event(listener['accountId'], on_reconnected_task(listener))
         except Exception as err:
             print(f'[{datetime.now().isoformat()}] Failed to process reconnected event', format_error(err))
 
