@@ -1,17 +1,25 @@
-from mock import AsyncMock, patch
+from mock import AsyncMock, patch, MagicMock
 import pytest
 from asyncio import sleep
 import asyncio
 from freezegun import freeze_time
+from datetime import datetime
 from .metaApiWebsocket_client import MetaApiWebsocketClient
 from .synchronizationThrottler import SynchronizationThrottler
 
 
 class MockClient(MetaApiWebsocketClient):
+    def __init__(self, token):
+        super().__init__(MagicMock(), token)
+        self._subscribed_account_ids = ['accountId1'] * 11
+
     async def _rpc_request(self, account_id: str, request: dict, timeout_in_seconds: float = None):
         await sleep(0.1)
         pass
 
+    def subscribed_account_ids(self, socket_instance_index=None):
+        return self._subscribed_account_ids
+
 
 start_time = '2020-10-05 10:00:00.000'
 throttler: SynchronizationThrottler = None
@@ -25,7 +33,7 @@ async def run_around_tests():
         client = MockClient('token')
         client._rpc_request = AsyncMock()
         global throttler
-        throttler = SynchronizationThrottler(client, 2)
+        throttler = SynchronizationThrottler(client, 0)
         throttler.start()
         yield
         throttler.stop()
@@ -70,6 +78,35 @@ class TestSynchronizationThrottler:
         await sleep(0.1)
         assert client._rpc_request.call_count == 3
 
+    @pytest.mark.asyncio
+    async def test_increase_slots_with_more_accounts(self):
+        """Should increase slot amount with more subscribed accounts."""
+        client._subscribed_account_ids = ['accountId1'] * 21
+        await throttler.schedule_synchronize('accountId1', {'requestId': 'test1'})
+        await throttler.schedule_synchronize('accountId2', {'requestId': 'test2'})
+        await throttler.schedule_synchronize('accountId3', {'requestId': 'test3'})
+        client._rpc_request.assert_any_call('accountId1', {'requestId': 'test1'})
+        client._rpc_request.assert_any_call('accountId2', {'requestId': 'test2'})
+        client._rpc_request.assert_any_call('accountId3', {'requestId': 'test3'})
+        assert client._rpc_request.call_count == 3
+
+    @pytest.mark.asyncio
+    async def test_limit_slots_in_options(self):
+        """Should set hard limit for concurrent synchronizations across throttlers via options."""
+        client._subscribed_account_ids = ['accountId1'] * 21
+        throttler = SynchronizationThrottler(client, 0, {'maxConcurrentSynchronizations': 3})
+        client._socketInstances = [{'synchronizationThrottler': throttler}, {'synchronizationThrottler': MagicMock()}]
+        client._socketInstances[1]['synchronizationThrottler'].synchronizing_accounts = ['accountId4']
+        await throttler.schedule_synchronize('accountId1', {'requestId': 'test1'})
+        await throttler.schedule_synchronize('accountId2', {'requestId': 'test2'})
+        asyncio.create_task(throttler.schedule_synchronize('accountId3', {'requestId': 'test3'}))
+        asyncio.create_task(throttler.schedule_synchronize('accountId4', {'requestId': 'test4'}))
+        await sleep(0.1)
+        assert client._rpc_request.call_count == 2
+        throttler.remove_synchronization_id('test1')
+        await sleep(0.1)
+        assert client._rpc_request.call_count == 3
+
     @pytest.mark.asyncio
     async def test_not_take_slots_if_same_account(self):
         """Should not take extra slots if sync ids belong to the same account."""
@@ -132,10 +169,10 @@ class TestSynchronizationThrottler:
         """Should clear existing sync ids on disconnect."""
         await throttler.schedule_synchronize('accountId1', {'requestId': 'test1'})
         await throttler.schedule_synchronize('accountId2', {'requestId': 'test2'})
-        asyncio.create_task(throttler.schedule_synchronize('accountId3', {'requestId': 'test3'}))
         await sleep(0.2)
         assert client._rpc_request.call_count == 2
         throttler.on_disconnect()
+        await throttler.schedule_synchronize('accountId3', {'requestId': 'test3'})
         await sleep(0.2)
         assert client._rpc_request.call_count == 3
 
@@ -195,3 +232,66 @@ class TestSynchronizationThrottler:
             client._rpc_request.assert_any_call('accountId1', {'requestId': 'test1'})
             client._rpc_request.assert_any_call('accountId2', {'requestId': 'test2'})
             client._rpc_request.assert_any_call('accountId5', {'requestId': 'test5'})
+
+    @pytest.mark.asyncio
+    async def test_should_not_get_stuck_due_to_app_limit(self):
+        """Should not get queue stuck due to app synchronizations limit."""
+        with patch('lib.clients.metaApi.synchronizationThrottler.asyncio.sleep', new=lambda x: sleep(x / 50)):
+            throttler._client._socketInstances = [{'synchronizationThrottler': MagicMock()},
+                                                  {'synchronizationThrottler': throttler}]
+            throttler._client.socket_instances[0]['synchronizationThrottler'].synchronizing_accounts = [
+                'accountId21', 'accountId22', 'accountId23', 'accountId24', 'accountId25', 'accountId26', 'accountId27',
+                'accountId28', 'accountId29', 'accountId210', 'accountId211', 'accountId212', 'accountId213',
+                'accountId214', 'accountId215']
+            asyncio.create_task(throttler.schedule_synchronize('accountId1', {'requestId': 'test1'}))
+            asyncio.create_task(throttler.schedule_synchronize('accountId2', {'requestId': 'test2'}))
+            asyncio.create_task(throttler.schedule_synchronize('accountId3', {'requestId': 'test3'}))
+            await sleep(0.11)
+            client._rpc_request.assert_not_called()
+            throttler._client.socket_instances[0]['synchronizationThrottler'].synchronizing_accounts = \
+                throttler._client.socket_instances[0]['synchronizationThrottler'].synchronizing_accounts[1:]
+            await sleep(0.11)
+            assert client._rpc_request.call_count == 1
+            throttler._client.socket_instances[0]['synchronizationThrottler'].synchronizing_accounts = \
+                throttler._client.socket_instances[0]['synchronizationThrottler'].synchronizing_accounts[1:]
+            await sleep(0.11)
+            assert client._rpc_request.call_count == 2
+
+    @pytest.mark.asyncio
+    async def test_should_not_skip_queue_items(self):
+        """Should not skip queue items when synchronization id is removed."""
+        with patch('lib.clients.metaApi.synchronizationThrottler.asyncio.sleep', new=lambda x: sleep(x / 20)):
+            asyncio.create_task(throttler.schedule_synchronize('accountId1', {'requestId': 'test1'}))
+            asyncio.create_task(throttler.schedule_synchronize('accountId2', {'requestId': 'test2'}))
+            asyncio.create_task(throttler.schedule_synchronize('accountId3', {'requestId': 'test3'}))
+            asyncio.create_task(throttler.schedule_synchronize('accountId4', {'requestId': 'test4'}))
+            asyncio.create_task(throttler.schedule_synchronize('accountId5', {'requestId': 'test5'}))
+            await sleep(0.1)
+            throttler.remove_synchronization_id('test3')
+            await sleep(0.1)
+            throttler.remove_synchronization_id('test1')
+            throttler.remove_synchronization_id('test2')
+            await sleep(0.1)
+            assert client._rpc_request.call_count == 4
+
+    @pytest.mark.asyncio
+    async def test_should_remove_id_by_parameters(self):
+        """Should remove id by parameters."""
+        await throttler.schedule_synchronize('accountId1', {'requestId': 'test1'})
+        await throttler.schedule_synchronize('accountId2', {'requestId': 'test2', 'instanceIndex': 0,
+                                                            'host': 'ps-mpa-0'})
+        asyncio.create_task(throttler.schedule_synchronize('accountId3', {'requestId': 'test3'}))
+        asyncio.create_task(throttler.schedule_synchronize('accountId2', {'requestId': 'test4', 'instanceIndex': 1,
+                                                           'host': 'ps-mpa-1'}))
+        asyncio.create_task(throttler.schedule_synchronize('accountId2', {'requestId': 'test5', 'instanceIndex': 0,
+                                                           'host': 'ps-mpa-2'}))
+        asyncio.create_task(throttler.schedule_synchronize('accountId4', {'requestId': 'test6'}))
+        await sleep(0.05)
+        throttler.remove_id_by_parameters('accountId2', 0, 'ps-mpa-0')
+        throttler.remove_id_by_parameters('accountId2', 1, 'ps-mpa-1')
+        throttler.remove_synchronization_id('test1')
+        await sleep(0.1)
+        client._rpc_request.assert_any_call('accountId3', {'requestId': 'test3'})
+        client._rpc_request.assert_any_call('accountId2', {'requestId': 'test5', 'instanceIndex': 0,
+                                                           'host': 'ps-mpa-2'})
+        assert client._rpc_request.call_count == 4
