@@ -6,6 +6,7 @@ import random
 import string
 import pytz
 import re
+import traceback
 
 
 def date(date_time: str or float or int) -> datetime:
@@ -49,6 +50,25 @@ def convert_iso_time_to_date(packet):
                             price['timestamps'][field] = date(price['timestamps'][field])
 
 
+def format_error(err: Exception):
+    """Formats and outputs metaApi errors with additional information.
+
+    Args:
+        err: Exception to process.
+    """
+    error = {'name': err.__class__.__name__, 'message': err if isinstance(err, str) else err.args[0]}
+    if hasattr(err, 'status_code'):
+        error['status_code'] = err.status_code
+    if err.__class__.__name__ == 'ValidationException':
+        error['details'] = err.details
+    if err.__class__.__name__ == 'TradeException':
+        error['string_code'] = err.stringCode
+    if err.__class__.__name__ == 'TooManyRequestsException':
+        error['metadata'] = err.metadata
+    error['trace'] = traceback.format_exc()
+    return error
+
+
 class MetatraderAccountInformation(TypedDict):
     """MetaTrader account information (see https://metaapi.cloud/docs/client/models/metatraderAccountInformation/)"""
 
@@ -140,7 +160,7 @@ class MetatraderPosition(TypedDict):
     POSITION_REASON_WEB, POSITION_REASON_UNKNOWN. See
     https://www.mql5.com/en/docs/constants/tradingconstants/positionproperties#enum_position_reason"""
     accountCurrencyExchangeRate: Optional[float]
-    """Current exchange rate of account currency into USD."""
+    """Current exchange rate of account currency into account base currency (USD if you did not override it)."""
     originalComment: Optional[str]
     """Position original comment (present if possible to restore from history)."""
     updatePending: Optional[bool]
@@ -218,7 +238,7 @@ class MetatraderOrder(TypedDict):
     expirationTime: datetime
     """Optional order expiration time."""
     accountCurrencyExchangeRate: Optional[float]
-    """Current exchange rate of account currency into USD."""
+    """Current exchange rate of account currency into account base currency (USD if you did not override it)."""
     closeByPositionId: Optional[str]
     """Identifier of an opposite position used for closing by order ORDER_TYPE_CLOSE_BY"""
     stopLimitPrice: Optional[float]
@@ -293,7 +313,7 @@ class MetatraderDeal(TypedDict):
     DEAL_REASON_SPLIT, DEAL_REASON_UNKNOWN. See
     https://www.mql5.com/en/docs/constants/tradingconstants/dealproperties#enum_deal_reason."""
     accountCurrencyExchangeRate: Optional[float]
-    """Current exchange rate of account currency into USD."""
+    """Current exchange rate of account currency into account base currency (USD if you did not override it)."""
 
 
 class MetatraderDeals(TypedDict):
@@ -420,8 +440,8 @@ class MetatraderSymbolSpecification(TypedDict):
     swapShort: Optional[float]
     """Short swap value."""
     swapRollover3Days: str
-    """Day of week to charge 3 days swap rollover. Allowed values are SUNDAY, MONDAY, TUESDAY, WEDNESDAY, THURDAY,
-    FRIDAY, SATURDAY."""
+    """Day of week to charge 3 days swap rollover. Allowed values are SUNDAY, MONDAY, TUESDAY, WEDNESDAY, THURSDAY,
+    FRIDAY, SATURDAY, NONE."""
     allowedExpirationModes: List[str]
     """Allowed order expiration modes. Allowed values are SYMBOL_EXPIRATION_GTC, SYMBOL_EXPIRATION_DAY,
     SYMBOL_EXPIRATION_SPECIFIED, SYMBOL_EXPIRATION_SPECIFIED_DAY.
@@ -464,7 +484,7 @@ class MetatraderSymbolPrice(TypedDict):
     lossTickValue: float
     """Tick value for a loosing position."""
     accountCurrencyExchangeRate: float
-    """Current exchange rate of account currency into USD."""
+    """Current exchange rate of account currency into account base currency (USD if you did not override it)."""
     time: str
     """Quote time, in ISO format"""
     brokerTime: str
@@ -565,3 +585,102 @@ class ExceptionMessage(TypedDict):
     """Human-readable error message"""
     details: Optional[List[ValidationDetails]]
     """Additional information about error. Used to supply validation error details."""
+
+
+class MarketDataSubscription(TypedDict):
+    """Market data subscription."""
+    type: str
+    """Subscription type, one of quotes, candles, ticks, or marketDepth."""
+    timeframe: Optional[str]
+    """When subscription type is candles, defines the timeframe according to which the candles must be generated.
+    Allowed values for MT5 are 1m, 2m, 3m, 4m, 5m, 6m, 10m, 12m, 15m, 20m, 30m, 1h, 2h, 3h,
+    4h, 6h, 8h, 12h, 1d, 1w, 1mn. Allowed values for MT4 are 1m, 5m, 15m 30m, 1h, 4h, 1d, 1w, 1mn."""
+    intervalInMilliseconds: Optional[float]
+    """Defines how frequently the terminal will stream data to client. If not set, then the value configured in
+    account will be used."""
+
+
+class MarketDataUnsubscription(TypedDict):
+    """Market data subscription."""
+    type: str
+    """Subscription type, one of quotes, candles, ticks, or marketDepth."""
+
+
+class MetatraderCandle(TypedDict):
+    """MetaTrader candle."""
+    symbol: str
+    """Symbol (e.g. currency pair or an index)."""
+    timeframe: str
+    """Timeframe candle was generated for, e.g. 1h. One of 1m, 2m, 3m, 4m, 5m, 6m, 10m, 12m, 15m, 20m, 30m, 1h, 2h,
+    3h, 4h, 6h, 8h, 12h, 1d, 1w, 1mn."""
+    time: datetime
+    """Candle opening time."""
+    brokerTime: str
+    """Candle opening time, in broker timezone, YYYY-MM-DD HH:mm:ss.SSS format."""
+    open: float
+    """Open price."""
+    high: float
+    """High price."""
+    low: float
+    """Low price."""
+    close: float
+    """Close price."""
+    tickVolume: float
+    """Tick volume, i.e. number of ticks inside the candle."""
+    spread: float
+    """Spread in points."""
+    volume: float
+    """Trade volume."""
+
+
+class MetatraderTick(TypedDict):
+    """MetaTrader tick data."""
+    symbol: str
+    """Symbol (e.g. a currency pair or an index)."""
+    time: datetime
+    """Time."""
+    brokerTime: str
+    """Candle opening time, in broker timezone, YYYY-MM-DD HH:mm:ss.SSS format."""
+    bid: Optional[float]
+    """Bid price."""
+    ask: Optional[float]
+    """Ask price."""
+    last: Optional[float]
+    """Last deal price."""
+    volume: float
+    """Volume for the current last deal price."""
+    side: str
+    """Side is tick a result of buy or sell deal, one of buy or sell."""
+
+
+class MetatraderBookEntry(TypedDict):
+    """MetaTrader order book entry."""
+    type: str
+    """Entry type, one of BOOK_TYPE_SELL, BOOK_TYPE_BUY, BOOK_TYPE_SELL_MARKET, BOOK_TYPE_BUY_MARKET."""
+    price: float
+    """Price."""
+    volume: float
+    """Volume."""
+
+
+class MetatraderBook(TypedDict):
+    """MetaTrader order book."""
+    symbol: str
+    """Symbol (e.g. a currency pair or an index)."""
+    time: datetime
+    """Time."""
+    brokerTime: str
+    """Candle opening time, in broker timezone, YYYY-MM-DD HH:mm:ss.SSS format."""
+    book: List[MetatraderBookEntry]
+    """List of order book entries."""
+
+
+class StopOptions(TypedDict):
+    """Stop options."""
+    value: float
+    """Stop (SL or TP) value."""
+    units: str
+    """stop units. ABSOLUTE_PRICE means the that the value of value field is a final stop value.
+    RELATIVE_* means that the value field value contains relative stop expressed either in price, points, account
+    currency or balance percentage. Default is ABSOLUTE_PRICE. Allowed values are ABSOLUTE_PRICE, RELATIVE_PRICE,
+    RELATIVE_POINTS, RELATIVE_CURRENCY, RELATIVE_BALANCE_PERCENTAGE."""
